/*-
 * Copyright (c) 2016 UPLEX Nils Goroll Systemoptimierung
 * All rights reserved
 *
 * Author: Geoffrey Simmons <geoffrey.simmons@uplex.de>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include "vre2.h"

#define CATCHALL                                \
        catch (const runtime_error& err) {      \
                return err.what();              \
        }                                       \
        catch (const exception& ex) {           \
                return ex.what();               \
        }                                       \
        catch (...) {                           \
                return "Unknown error";         \
        }

using namespace std;

vre2::vre2(const char *pattern) {
	re_ = new RE2(pattern, RE2::Quiet);
	if (!re_->ok())
		throw runtime_error(re_->error());
}

vre2::~vre2() {
	if (re_) {
		delete re_;
		re_ = NULL;
	}
}

inline bool
vre2::match(const char *subject, int ngroups, StringPiece* groups)
{
	return re_->Match(subject, 0, strlen(subject), RE2::UNANCHORED,
	    groups, ngroups + 1);
}

inline int
vre2::ngroups()
{
	return re_->NumberOfCapturingGroups();
}

const char *
vre2_init(vre2 **vre2p, const char *pattern)
{
	try {
		*vre2p = new vre2(pattern);
		return NULL;
	}
	CATCHALL
}

const size_t
vre2_matchsz(void)
{
	return sizeof(StringPiece);
}

const char *
vre2_match(vre2 *vre2, const char *subject, int *match, int ngroups,
    void *group)
{
	try {
		StringPiece* g = reinterpret_cast<StringPiece *>(group);
		*match = vre2->match(subject, ngroups, g);
		return NULL;
	}
	CATCHALL
}

const char *
vre2_capture(void *group, int refnum, const char **capture, int *len)
{
	try {
		StringPiece* g = reinterpret_cast<StringPiece *>(group);
		StringPiece str = g[refnum];
		*capture = str.data();
		*len = str.length();
		return NULL;
	}
	CATCHALL
}

const char *
vre2_ngroups(vre2 *vre2, int *ngroups)
{
	try {
		*ngroups = vre2->ngroups();
		return NULL;
	}
	CATCHALL
}

const char *
vre2_fini(vre2 **vre2)
{
	try {
		delete *vre2;
		return NULL;
	}
	CATCHALL
}
